


## Converts a tipical groyp by lor with A, B, value, value into an other lod with A as rows, B as columns and value as AxB list of dic
## columns order can be defined in order
def lod_xyv_transformation(ld, key_x, key_y, key_value, order=None):
    if len(ld)==0:
       return []

    if not key_x in ld[0] or not key_y in ld[0] or not key_value in ld[0]:
        print("Keys names are not correct in dictionary in lod_year_month_value_transposition function")
        return None

    #Searches for all diferent keys
    columns=set()
    rows=set()
    for d in ld:
        columns.add(d[key_y])
        rows.add(d[key_x])
    columns=list(columns)
    rows=list(rows)
    
    #Initialize result with a dictionary of dictionary
    dd={}
    for row in rows:
        d={"title": row}
        for column in columns:
            d[column]=0
        dd[row]=d

    #Assign values
    for d in ld:
        dd[d[key_x]][d[key_y]]=d[key_value]
    
    ## Converts dd to a ld
    r=[]
    for k, v in dd.items():
        r.append(v)
    
    return r



from pydicts import lod_ymv

def tests_lod_year_month_value_transposition():
    o=[
        {"year": 2022, "month": 1, "my_sum": 12},
        {"year": 2021, "month": 2, "my_sum": 123},
        {"year": 2019, "month": 5, "my_sum": 1},
        {"year": 2022, "month": 12, "my_sum": 12},
    ]
    t=lod_ymv.lod_ymv_transposition(o,key_value="my_sum")
    assert t[0]["year"]==2019
    assert t[3]["total"]==24



from datetime import datetime, date
from decimal import Decimal
from pydicts import lod
from pytest import raises

empty_lod=[]

lod_=[]
lod_.append({"a": datetime.now(), "b": date.today(), "c": Decimal('12.32'), "d": None, "e": int(12), "f":None, "g":True, "h":False})
lod_.append({"a": datetime.now(), "b": date.today(), "c": Decimal('-12.32'), "d": 16, "e": int(12), "f":None, "g":True, "h":False})


def tests_dictkv():
    d=lod.lod2dictkv(lod_, "a","b")
    assert list(d.values())[0]==date.today()
    

def tests_lod_has_key():
    assert lod.lod_has_key(lod_,"c")==True
    assert lod.lod_has_key(lod_,"cc")==False
    assert lod.lod_has_key(empty_lod, "c")==False
    
def tests_lod_sum():
    assert lod.lod_sum(lod_, "c")==0
    assert lod.lod_sum(lod_, "d")==16
    with raises(TypeError) as excinfo:
        lod.lod_sum(lod_, "d", ignore_nones=False)==16
    assert "NoneType" in str(excinfo.value)
    
def test_lod_remove_key():
    assert lod.lod_has_key(lod_, "d")
    lod.lod_remove_key(lod_, "d")
    assert not lod.lod_has_key(lod_, "d")
    



"""
    This class is used with the pylatex module to generate table from pydicts structures
    You only have to import pylatex if you're testing
"""


from pylatex import LongTabularx, MultiColumn
from pylatex.basic import NewLine
from pylatex.utils import NoEscape, bold, escape_latex
from pydicts import lod
def pylatex_table(
    doc, 
    lod_, 
    code_=None, 
    text_no_results="No data to show", 
):
    """
    Creates a table in a pylatex document

    @param doc pylatex document object
    @param lod 
    @param code_ |c|r|l|
    """
    
    keys= lod.lod_keys(lod_)
    if keys is None:
        doc.append(text_no_results)
        doc.append(NewLine())
        return
        
    headers=[]
    for key in keys:
        headers.append(NoEscape(bold(key)))
    
    # Generate data table
    number=0.9/len(headers)
    if code_ is None:
        code_=f"|p{{{number}\\linewidth}}"*len(headers)+"|"
        
    with doc.create(LongTabularx(code_)) as data_table:
        data_table.add_hline()
        data_table.add_row(headers)
        data_table.add_hline()
        data_table.end_table_header()
        data_table.add_hline()
        data_table.add_row((MultiColumn(len(headers), align='r', data='La tabla continúa en la siguiente página'),))
        data_table.end_table_footer()
        data_table.end_table_last_footer()

        escaped_list=[]
        for list_ in lod.lod2lol(lod_):
            row=[]
            for column in range(len(headers)):
                row.append(escape_latex(list_[column]))#Escape values
            escaped_list.append(row)

        for list_ in escaped_list:
            data_table.add_row(list_)
            data_table.add_hline()

    
def pylatex_table_with_matched_values(
    doc, 
    values_to_match, 
    lod_, 
    code_=None, 
    text_no_results="No data to show", 
    match_color="teal", 
    unmatch_color="red"
):
    """
    Creates a table in a pylatex document
    
    This function needs to have xcolor package loaded in document with
    doc.packages.append(Package('xcolor'))

    @param doc pylatex document object
    @param values_to_match Values to match is a list
    @param lod 
    @param code_ |c|r|l|   Para usar wrapping |p{.20\\linewidth\p{.80\\linewidth}|
    """
    keys= lod.lod_keys(lod_)
    if keys is None:
        doc.append(text_no_results)
        doc.append(NewLine())
        return
        
    headers=[]
    for key in keys:
        headers.append(NoEscape(bold(key)))
  
    # Generate data table
    number=0.9/len(headers)
    if code_ is None:
        code_=f"|p{{{number}\\linewidth}}"*len(headers)+"|"
        
    with doc.create(LongTabularx(code_)) as data_table:
        data_table.add_hline()
        data_table.add_row(headers)
        data_table.add_hline()
        data_table.end_table_header()
        data_table.add_hline()
        data_table.add_row((MultiColumn(len(headers), align='r', data='La tabla continúa en la siguiente página'),))
        data_table.end_table_footer()
        data_table.end_table_last_footer()

        #Prepare cells
        matched_list=[]
        for list_ in lod.lod2lol(lod_):
            row=[]
            for column in range(len(headers)):
                if list_[column]==values_to_match[column]:
                    row.append(NoEscape(f"\\textcolor{{{match_color}}}{{{escape_latex(list_[column])}}}"))
                else:
                    row.append(NoEscape(f"\\textcolor{{{unmatch_color}}}{{{escape_latex(list_[column])}}}"))
            matched_list.append(row)
                
        
        for list_ in matched_list:
            data_table.add_row(list_)
            data_table.add_hline()
            

#    doc.append(NewLine())



from pydicts import pylatex
from pylatex import Document, Section, Subsection
from pylatex.package import Package
from pylatex.utils import italic, NoEscape
lod=[
        {"year": 2022, "month": 1, "my_sum": 12},
        {"year": 2021, "month": 2, "my_sum": 123},
        {"year": 2019, "month": 5, "my_sum": 1},
        {"year": 2022, "month": 12, "my_sum": 12},
]
def tests_pylatex_table_header():

    doc = Document()
    doc.packages.append(Package('xcolor'))

    with doc.create(Section('A section')):
        doc.append('Some regular text and some ')
        doc.append(italic('italic text. '))

        with doc.create(Subsection('A subsection')):
            doc.append('Also some crazy characters: $&#{}')
            
    doc.append(NoEscape("\\centering"))
    pylatex.pylatex_table(doc, lod)
    pylatex.pylatex_table(doc, [])
    pylatex.pylatex_table_with_matched_values(doc,  [2022, 2, 12], lod, code_="|l|c|r|", match_color="teal", unmatch_color="red")
    doc.generate_pdf('test_pylatex_table_header', clean_tex=False)
